%
% Example_Mesh: Creates a mesh of a root system using DistMesh
%
% Before running copy Per-Olof Persson's DistMesh from
% http://www-math.mit.edu/~persson/mesh/
% into the same folder and set ptol=0.1 and ttol=0.01 in the file
% distmeshnd.m, line 24
%
% Meshing works only for rather small root systems with not too small root
% radius. Meshing takes up to half an hour.
%
% See also: Example_Maize, distmeshnd, distSegment.
%
% Copyright 2008-2009 A. Schnepf, D. Leitner. See license.txt for details.
%   

%
% creates a single 12 days old maize root 
%
% 1st Order
p{2}.r = 6.4; p{2}.rs = 0; p{2}.lb = 0.7; p{2}.lbs = 0; p{2}.la = 2; 
p{2}.las = 0; p{2}.ln = 0.7; p{2}.lns = 0; p{2}.nob = 7; p{2}.nobs = 0;                 
p{2}.a = 4*0.02; p{2}.as = 0; p{2}.theta = 63/360*2*pi; 
p{2}.thetas = 15/360*2*pi; p{2}.color = [140/255,125/255,40/255]; 
p{2}.N = 2; p{2}.sigma = 20*pi/180;   
% 0th Order
p{1}.r = 2; p{1}.rs = 0; p{1}.lb = 0.7; p{1}.lbs = 0; p{1}.la = 15.7; 
p{1}.las = 0; p{1}.ln = 0.7; p{1}.lns = 0; p{1}.nob = 50; p{1}.nobs = 0;   
p{1}.a = 4*0.04; p{1}.as = 0; p{1}.color = [110/255,100/255,50/255];
p{1}.N = 1.5; p{1}.sigma = 10*pi/180;  
dx = 0.1;
root = createBranchedRoot(p, dx); 
n0 = 1;  r0 = 0;  
rootsystem = createRootSystem(root,n0,r0);
slides = simulate(rootsystem,12);

%
% create a mesh using DistMesh
%
bnd = getBounds(slides{end});
[x1_,x2_,r_,times_] = getSegments(slides{end});
roots  = @(p) distSegment(p,x1_,x2_,r_);
bndbox = @(p) dblock(p,bnd(1),bnd(2),bnd(3),bnd(4),bnd(5),bnd(6));
fd = @(p,arg) ddiff(bndbox(p),roots(p)); % signed distance function
fh = @(p,arg) min(abs(roots(p))*0.5+0.05,0.5); % mesh size function

bndp = [bnd(1),bnd(3),bnd(5); bnd(2),bnd(4),bnd(6) ];
fixp = [bnd(1),bnd(3),bnd(5); bnd(2),bnd(3),bnd(5); bnd(1),bnd(4),bnd(5); ...
    bnd(2),bnd(4),bnd(5); bnd(1),bnd(3),bnd(6); bnd(2),bnd(3),bnd(6);...
    bnd(1),bnd(4),bnd(6); bnd(2),bnd(4),bnd(6) ]; % fix points

disp('DistMesh');
tic
[nodes, tetra]=distmeshnd(fd,fh,0.07,bndp,fixp,[]);
toc
%
% plot mesh
%
figure
t=surftri(nodes,tetra);
trimesh(t, nodes(:,1),nodes(:,2),nodes(:,3) )
axis(bnd)
axis equal
alpha 0.5

% %
% % convert to Comsol mesh (works only for very small examples due to
% % mysterious Comsol geometry errors) 
% %
% el = cell(1,0);
% el{1} = struct('type','tet','elem',tetra');
% m = femmesh(nodes',el);
% m = meshenrich(m);
% clear fem
% fem.mesh = m;
%



