%% 3D ROOT SYSTEM SIMULATION aroot_simu.m
%% Imperial College London iGem 2011
%% Nina Jiayue Zhu and Si Chen
%
% This a 3D visualisation program for root systems with root order less
% than 3, developed from the idealized maize root system modelling done by BOKU, Vienna.
% The parameters are given for each order of the root system:
% r:            initial growth rate (cm/day)
% The initial growth rate is determined by the input IAA pushbutton2 and
% the calculation is derived from the data fitting experiment done by
% Imperial College London iGem 2011 Team for Thaliana Arabidopsis.
% la, lb, ln:   length of apical, basal and inter-root distance (cm)
% nob:          number of lateral branches (-)
% k:            maximal length (for highest order roots only instead of la,
%       lb, ln, nob) (cm)
% a:            radius (cm)
% theta:        angle between root and later root of next order (-)
% color:        color of the root (RGB)
% N:            strength of gravitropism (-)
% sigma:        additional random noise to the heading (cm-1)
% bend:         an arbitrary deflection rule (instead of N and sigma)
%       (Matlab function) 
% An additional s in the name denotes the standard deviation of each
% parameter.

%% GUI design
function varargout = aroot_simu(varargin)
% AROOT_SIMU MATLAB code for aroot_simu.fig
%      AROOT_SIMU, by itself, creates a new AROOT_SIMU or raises the existing
%      singleton*.
%
%      H = AROOT_SIMU returns the handle to a new AROOT_SIMU or the handle to
%      the existing singleton*.
%
%      AROOT_SIMU('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in AROOT_SIMU.M with the given input arguments.
%
%      AROOT_SIMU('Property','Value',...) creates a new AROOT_SIMU or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before aroot_simu_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to aroot_simu_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help aroot_simu

% Last Modified by GUIDE v2.5 25-Oct-2011 18:29:34

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @aroot_simu_OpeningFcn, ...
                   'gui_OutputFcn',  @aroot_simu_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before aroot_simu is made visible.
function aroot_simu_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to aroot_simu (see VARARGIN)

% Choose default command line output for aroot_simu
handles.output = hObject;
num = 0;
handles.concentration = num;

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes aroot_simu wait for user response (see UIRESUME)
% uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = aroot_simu_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


function concentration_Callback(hObject, eventdata, handles)
% hObject    handle to concentration (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
num = str2double(get(hObject,'String'))
if isnan(num)
    num =0;
    set(hObject,'String',num);
    errordlg('You must type in a number','Error')
end
handles.concentration = num;
guidata(hObject,handles)
% Hints: get(hObject,'String') returns contents of concentration as text
%        str2double(get(hObject,'String')) returns contents of concentration as a double


% --- Executes during object creation, after setting all properties.
function concentration_CreateFcn(hObject, eventdata, handles)
% hObject    handle to concentration (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function pushbutton2_Callback(hObject, eventdata, handles)
% hObject    handle to concentration (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

auxin = handles.concentration;

%% simulation of a root system 

if  auxin == 0
    y = 0.03;
    ms = msgbox('Your plant will grow without additional IAA.','Simulation successful');
    fs = findobj( ms, 'Type', 'Text' );
    set(fs,'FontSize',18 );
    set(ms,'Position',[150 100 350 60] ); % set new position
    guidata(hObject,handles);
else
    x = log10(auxin);
    if x < -14
        ms = msgbox('The IAA concentration is too low. ','Simulation failed');
        fs = findobj( ms, 'Type', 'Text' );
        set(fs, 'FontSize',18 );
        set(ms, 'Position',[150 100 280 60] ); % set new position
        guidata(hObject,handles);
        return;
    elseif x > -3
         ms = msgbox('The IAA concentration is too high. ','Simulation failed');
        fs = findobj(ms, 'Type', 'Text' );
        set(fs, 'FontSize',18 );
        set(ms, 'Position',[150 100 280 60] );
        guidata(hObject,handles);
           return;
    else
        ms = msgbox('The root system after 25 days is shown. ','Simulation successful');
        fs = findobj(ms, 'Type', 'Text' );
        set(fs, 'FontSize',18 );
        set(ms, 'Position',[150 100 330 60] ); 
        guidata(hObject,handles);
        y = 4.833*exp(-((x+12.61)/ 0.5612)^2) + 1.31*exp(-((x+9.985)/4.666)^2);
    end
end

% % 2nd Order 
% p{3}.r  = 1.5;                       
% p{3}.rs = 0.15;                      
% p{3}.k  = 0.2;                    
% p{3}.ks = 0.02;            
% p{3}.a  = 0.01;                
% p{3}.as = 0;                   
% p{3}.theta = 68*pi/180;        
% p{3}.thetas = 3*pi/180;  
% p{3}.color = [200/255,200/255,10/255];  
% p{3}.N = 0;                         
% p{3}.sigma = 40*pi/180;

% 1st Order
p{2}.r = 0.9*y;
p{2}.rs = 0.014*(p{2}.r); 
p{2}.lb = 0.7;    
p{2}.lbs = 0.07;               
p{2}.la = 9.7;            
p{2}.las = 0.97;                     
p{2}.ln = 0.7;               
p{2}.lns = 0.07;       
p{2}.nob = 3;                  
p{2}.nobs =0.07;                 
p{2}.a = 0.03;                
p{2}.as = 0;                   
p{2}.theta = 50*pi/180;       
p{2}.thetas = 0.3*pi/180;
p{2}.color = [220/255,220/255,220/255];
p{2}.N = 2; 
p{2}.sigma = 15*pi/180;  

% 0th Order
p{1}.r = y;
p{1}.rs = 0.014*y;
p{1}.lb = 0.7;          
p{1}.lbs = 0.07;             
p{1}.la = 5.7;       
p{1}.las = 0.57;                  
p{1}.ln = 0.7;                 
p{1}.lns = 0.07; 
p{1}.nob = 20;  
p{1}.nobs = 0.1;   
p{1}.a = 0.1;                 
p{1}.as = 0;                    
p{1}.color = [210/255,200/255,210/255];
p{1}.N = 20; 
p{1}.sigma = 60*pi/180;  

figure
dx = 0.25;  
root = createBranchedRoot(p, dx); 
hold on
n0 = 1;
r0 = 5;
rootsystem = createRootSystem(root,n0,r0); 

%% animation production
days = 25;   % the simulation time (days)
steps = 10;
NextSlide = simulate(rootsystem,days,steps,false);
plotTubes(NextSlide{end});

% animate('root.avi',rootsystem,25,10,[-6,6,-6,6,-14,1,0,1]);
