function rootsystem = createRootSystem(root, number, r, e)
% createRootSystem: Creates a number of 0th order roots. 
%
% Parameters:
% root           :0th order root
% number         :initial number of roots
% r              :initial radius of growth cone  
% (e)            :exponent (0.5 for uniform distribution, >0.5 more inner,
%       <0.5 more outer) (default=0.5)
%
% rootsystem     :the resulting root system
%
% See also: Example_initialRoots1, Example_initialRoots2.
%
% Copyright 2008-2009 Daniel Leitner. See license.txt for details.
%     

if (nargin<4) 
    e = 0.5;
end

rules = getTurtle;

rules.BranchRotate.delta = atan(rand^e*r);
rules.PitchU.delta=pi/2;
rootsystem = [{rules.PitchU},{rules.Push},{rules.BranchRotate}, root, {rules.Pop}];

for i=1:number-1;
    rules.BranchRotate.delta = atan(rand^e*r);
    rootsystem = [rootsystem,{rules.Push}, {rules.BranchRotate}, root, {rules.Pop}];
end

rootsystem = initLabels(rootsystem);

