function [x1,x2,r,color,time]=getSegments(str, initial)
% getSegments: Gets the segments, radii and attributes of a root system
%
% Parameters:
% str           :List of L-System letters
% (initial)     :Initial values of the Turtle. Default: initial.xT=0,
%       initial.yT=0, initial.zT=0, initial.wT=0.1, initial.cT = [0,1,0],
%       R=diag([1,1,1])
%
% x1            :Start points of the segments
% x2            :End points of the segments
% r             :Radii of the segments
% color         :Colors of the segments
% time          :Creation times of the segments
%
% See also: getPolylines, drawLines.
%
% Copyright 2008-2009 Daniel Leitner. See license.txt for details.
%

if (nargin<2)  
    xT = 0;
    yT = 0;
    zT = 0;
    wT = 0.1;
    cT = [0, 1, 0];
    Rbk = [ cos(pi) sin(pi) 0;...
        -sin(pi) cos(pi) 0; ...
        0       0      1];
    R = diag([1 1 1]);
else
    xT = initial.xT;
    yT = initial.yT;
    zT = initial.zT;    
    wT = initial.wT;
    cT = initial.cT;
    R = initial.R;
end
stkPtr = 1;

c=1;
N = getTotal(str,'f');
x1 = zeros(N,3);
x2 = zeros(N,3);
r = zeros(N,1);
color = zeros(N,3);
time = zeros(N,1);

for i=1:length(str)
    
    cmdT = str{i}.name;
    
    switch cmdT
        case '#'
            wT = str{i}.width;        
        case 'C'
            cT = str{i}.color;
        case {'F'}
            lenF = str{i}.length;
            if isfield(str{i},'time')    
                t = str{i}.time;
                time(c) = t;
            end            
            newxT = xT + lenF*R(1,1);
            newyT = yT + lenF*R(2,1);
            newzT = zT + lenF*R(3,1);
            x1(c,1) = xT;
            x1(c,2) = yT;
            x1(c,3) = zT;
            x2(c,1) = newxT;
            x2(c,2) = newyT;            
            x2(c,3) = newzT;            
            r(c) = wT/2;
            color(c,:)= cT;
            xT = newxT;
            yT = newyT;
            zT = newzT;
            c=c+1;
        case '+'
            a = str{i}.delta;
            Rup = [    cos(a) sin(a) 0;...
                -sin(a) cos(a) 0; ...
                0       0      1];
            R2 = R * Rup;
            R = R2;
        case '-'
            a = str{i}.delta;
            Rum = [    cos(-a) sin(-a) 0;...
                -sin(-a) cos(-a) 0; ...
                0       0      1];
            R2 = R * Rum;
            R = R2;
        case '&'
            a = str{i}.delta;
            Rlp = [ cos(a) 0  -sin(a) ;...
                0          1     0; ...
                sin(a)    0   cos(a)];
            R2 = R * Rlp;
            R = R2;
        case '^'
            a = str{i}.delta;
            Rlm = [    cos(-a) 0  -sin(-a) ;...
                0          1     0; ...
                sin(-a)    0   cos(-a)];
            R2 = R * Rlm;
            R = R2;
        case '\'
            a = str{i}.delta;
            Rhp = [  1     0     0; ...
                0 cos(a) -sin(a) ;...
                0 sin(a)  cos(a) ] ;
            R2 = R * Rhp;
            R = R2;
        case '/'
            a = str{i}.delta;
            Rhm = [  1     0     0; ...
                0 cos(-a) -sin(-a) ;...
                0 sin(-a)  cos(-a) ] ;
            R2 = R * Rhm;
            R = R2;
        case '|'
            R2 = R * Rbk;
            R = R2;
        case '[' %push the stack
            stack(stkPtr).xT = xT ;
            stack(stkPtr).yT = yT ;
            stack(stkPtr).zT = zT ;
            stack(stkPtr).wT = wT;
            stack(stkPtr).cT = cT;
            stack(stkPtr).R = R;
            stkPtr = stkPtr +1 ;

        case ']' %pop the stack
            stkPtr = stkPtr -1;
            xT = stack(stkPtr).xT ;
            yT = stack(stkPtr).yT ;
            zT = stack(stkPtr).zT ;
            wT = stack(stkPtr).wT;
            cT = stack(stkPtr).cT;
            R = stack(stkPtr).R;
    end

end

