function x=labelTip(x,str)
% labelTip: Updates the position, width, color and heading of a letter x.
%
% The uptdate is performed done by applying the turtle commands given in
% str. 
%        
% Parameters:     
% x         :updates the parameters x.xT, x.yT, x.zT, x.wT, x.cT and x.R
% str       :List of L-System letters effecting tip postion and heading 
%
% x         :the labeled letter x.
%
% See also: initLabels.
%
% Copyright 2008-2009 Daniel Leitner. See license.txt for details.
%

stkPtr = 1;

for i=1:length(str)

    cmdT = str{i}.name;

    switch cmdT
        case '#'
            x.wT = str{i}.width;
        case 'C'
            x.cT = str{i}.color;
        case {'F', 'G'}
            lenF = str{i}.length;
            newxT = x.xT + lenF*x.R(1,1);
            newyT = x.yT + lenF*x.R(2,1);
            newzT = x.zT + lenF*x.R(3,1);
            x.xT = newxT;
            x.yT = newyT;
            x.zT = newzT;
        case '+'
            a = str{i}.delta;
            Rup = [    cos(a) sin(a) 0;...
                -sin(a) cos(a) 0; ...
                0       0      1];
            R2 = x.R * Rup;
            x.R = R2;
        case '-'
            a = str{i}.delta;
            Rum = [    cos(-a) sin(-a) 0;...
                -sin(-a) cos(-a) 0; ...
                0       0      1];
            R2 = x.R * Rum;
            x.R = R2;
        case '&'
            a = str{i}.delta;
            Rlp = [ cos(a) 0  -sin(a) ;...
                0          1     0; ...
                sin(a)    0   cos(a)];
            R2 = x.R * Rlp;
            x.R = R2;
        case '^'
            a = str{i}.delta;
            Rlm = [    cos(-a) 0  -sin(-a) ;...
                0          1     0; ...
                sin(-a)    0   cos(-a)];
            R2 = x.R * Rlm;
            x.R = R2;
        case '\'
            a = str{i}.delta;
            Rhp = [  1     0     0; ...
                0 cos(a) -sin(a) ;...
                0 sin(a)  cos(a) ] ;
            R2 = x.R * Rhp;
            x.R = R2;
        case '/'
            a = str{i}.delta;
            Rhm = [  1     0     0; ...
                0 cos(-a) -sin(-a) ;...
                0 sin(-a)  cos(-a) ] ;
            R2 = x.R * Rhm;
            x.R = R2;
        case '[' %push the stack
            stack(stkPtr).xT = x.xT ;
            stack(stkPtr).yT = x.yT ;
            stack(stkPtr).zT = x.zT ;
            stack(stkPtr).wT = x.wT;
            stack(stkPtr).cT = x.cT;
            stack(stkPtr).R = x.R;
            stkPtr = stkPtr +1 ;

        case ']' %pop the stack
            stkPtr = stkPtr -1;
            x.xT = stack(stkPtr).xT ;
            x.yT = stack(stkPtr).yT ;
            x.zT = stack(stkPtr).zT ;
            x.wT = stack(stkPtr).wT;
            x.cT = stack(stkPtr).cT;
            x.R = stack(stkPtr).R;
    end

end


