function new = ruleChemotropismN(y,N,sigma,dx,xx,yy,zz)
% ruleChemotropismN: The tendency grow towards higher nutrient concentration.
%
% Uses the global variable concentration to look up the concentration.
%
% Parametrs:
% y         :labeled lettter (containg position, heading, color and width)
% N         :N random tries to find best heading
% sigma     :additional random noise to the heading
% dx        :spatial discretization along the root
% xx,yy,zz  :partition of the domain (for concentration lookup)
%
% new       :resulting string
%
% See also: ruleTropismN, interp3.
%
% Copyright 2008-2009 Daniel Leitner. See license.txt for details.
%
c = 1e7;
g=0;

objfun = @(beta, alpha, y) test(beta, alpha,y,c,g,xx,yy,zz);
new = ruleTropismN(y,N,sigma,dx,objfun);


function z=test(r,t,y,c,g,xx,yy,zz)
global concentration;
%roll left
Rhp = [  1     0     0; ...
    0 cos(r) -sin(r) ;...
    0 sin(r)  cos(r) ] ;
R2 = y.R * Rhp;
%turn
Rum = [    cos(t) sin(t) 0;...
    -sin(t) cos(t) 0; ...
    0       0      1];
R3 = R2 * Rum;

dx=0.5;

x_ = y.xT+dx*R3(1,1);
y_ = y.yT+dx*R3(2,1);
z_ = y.zT+dx*R3(3,1);
s= interp3(xx,yy,zz,concentration,y_,x_,z_);

z = -c*s + g*R3(3,1);

