function new = ruleGrowth(x,lambda,deflect,dx,dt)
% ruleGrowth: Describes growth of a single root without branchting
% 
% Parameters:
% x         :letter that is replaced by the production rule
% -> x.t    :local age
% -> x.l    :approximation of the actual root length
% lambda    :growth function lambda(t)
% deflect   :production rule for root tip deflection
% dx        :spatial discretization along the root 
% dt        :time step
%
% new       :resulting string
%
% See also: ruleSectionGrowth.
%
% Copyright 2008-2009 Daniel Leitner. See license.txt for details.
%

if (x.l+dx<lambda(x.t+dt,x)) % new segments

    f.name='F';
    f.time = 0; % time is not supported (use ruleSectionGrowth)
    f.length=dx;
    x.l = x.l + dx;
    bend = deflect(x,dx);
    x=labelTip(x,[bend, {f}]);
    new = [ bend, {f}, x.apply(x,dt) ];   
    
else % do nothing

    x.t = x.t + dt;
    new  = { x };

end

