function new = ruleSectionGrowth(x,lambda,l0,lend,t0,tend,next,deflect,dx,dt)
% ruleSectionGrowth: Describes root growth within a section.
%
% Parameters:
% x         :letter that is replaced by the production rule
% -> x.t    :local age
% -> x.l    :approximation of the actual root length
% lambda    :growth function lambda(t0+t)
% t0        :creation time
% tend      :terminal time
% next      :successive string
% deflect   :production rule for root tip deflection
% dx        :spatial discretization along the root 
% dt        :time step
%
% new       :resulting string
%
% See also: ruleCreateRoot, ruleCreateHighestOrderRoot.
%
% Copyright 2008-2009 Daniel Leitner. See license.txt for details.
%

ls = lend-l0;
A=min(l0+x.l+dx,lend) <= lambda(t0+x.t+dt,x);

if  A && (x.l+dx < ls) % new segments

    f.name='F';
    f.length=dx;
    f.time = t0 + x.t; 
    x.l = x.l + dx;
    bend=deflect(x,dx);
    x=labelTip(x,[bend, {f}]);
    new = [bend, {f}, x.apply(x,dt) ];  %

elseif A && (x.l+dx >= ls) % end
    
    lr = ls-x.l;
    ct = t0 + x.t + dt - tend; % Initialize Next with ct

    if (lr>0)
        f.name = 'F';
        f.length=lr;
        f.time = t0 + x.t; 
        bend = deflect(x,lr);
        x=labelTip(x,[bend,{f}]);
        if isfield(x,'xT') % label next
            next.xT = x.xT;
            next.yT = x.yT;
            next.zT = x.zT;
            next.wT = x.wT;
            next.cT = x.cT;
            next.R = x.R;
        end
        new = [bend,{f},next.apply(next,ct)];
    else
        new=next.apply(next,ct);
    end
    
else % do nothing
    
    x.t = x.t+dt;
    new = {x};

end

