function new = ruleTropismN(y,N,sigma,dx,objfun)
% ruleTropism: Root tip deflection, favouring headings where objfun becomes small.
%
% Parameters:
% y         :labeled lettter (containg position, heading, color and width)
% N         :N random tries to find best heading
% sigma     :additional random noise to the heading
% dx        :spatial discretization along the root
% objfun    :objfun(alpha,beta,y), objective function
%
% new       :resulting string
%
% See also: ruleGravitropismN, rulePlagiotropismN, ruleChemotropismN
%
% Copyright 2008-2009 Daniel Leitner. See license.txt for details.
%

% find heading
if N>0

    N = N * sqrt(dx);
    dn = N-floor(N);
    if dn>0
        if rand()<dn
            N=ceil(N);
        else
            N=floor(N);
        end
    end
    
    beta= rand*2*pi;
    rr = beta;    
    alpha = sigma*randn*sqrt(dx);
    tt = alpha;
    zm = objfun(beta,alpha,y);

    for i=1:N
        beta = rand*2*pi;
        alpha = sigma*randn*sqrt(dx);
        z=objfun(beta,alpha,y);
        if (z<zm) %*vz
            zm=z;
            rr=beta;
            tt=alpha;            
        end
    end
    beta = rr;
    alpha = tt;    
    
else

    beta = pi-rand*2*pi;
    alpha =sigma*rand*sqrt(dx);

end

% Create rules
new = {};
if (alpha~=0)
    if (beta~=0)
        r.name = '\';
        r.delta = beta;
        new = [new,{r}];
    end
    t.name = '+';
    t.delta = alpha;
    new = [new,{t}];
end

